#ifndef _ALPHADATA_ADXDMA_BC_VERSION_H
#define _ALPHADATA_ADXDMA_BC_VERSION_H

#define ADXDMA_BC_H_MAKE_VERSION(super, major, minor) ((super) * 65536 + (major) * 256 + (minor))
#define ADXDMA_BC_H_VERSION_SUPER (0)
#define ADXDMA_BC_H_VERSION_MAJOR (2)
#define ADXDMA_BC_H_VERSION_MINOR (0)
#define ADXDMA_BC_H_VERSION ADXDMA_H_MAKE_VERSION(ADXDMA_BC_H_VERSION_SUPER, ADXDMA_BC_H_VERSION_MAJOR, ADXDMA_BC_H_VERSION_MINOR)

#endif
